@@ -2,19 +2,19 @@
 /*
  * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
- * Authors: Michael Zucchi <notzed@ximian.com>
- *
- * This library is free software you can redistribute it and/or modify it
+ * This library is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  * for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Michael Zucchi <notzed@ximian.com>
  */
 
 #if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
@@ -290,7 +290,7 @@ struct _CamelFolderSummaryClass {
 					 const gchar *bodystructure);
 	void		(*message_info_free)
 					(CamelFolderSummary *summary,
-					 CamelMessageInfo *info);
+					 CamelMessageInfo *ci);
 	CamelMessageInfo *
 			(*message_info_clone)
 					(CamelFolderSummary *summary,
@@ -311,7 +311,7 @@ struct _CamelFolderSummaryClass {
 					 CamelMimePart *mime_part);
 	void		(*content_info_free)
 					(CamelFolderSummary *summary,
-					 CamelMessageContentInfo *info);
+					 CamelMessageContentInfo *ci);
 	CamelMessageInfo *
 			(*message_info_from_uid)
 					(CamelFolderSummary *summary,
@@ -468,6 +468,9 @@ gboolean	camel_folder_summary_check_uid	(CamelFolderSummary *summary,
 CamelMessageInfo *
 		camel_folder_summary_get	(CamelFolderSummary *summary,
 						 const gchar *uid);
+guint32		camel_folder_summary_get_info_flags
+						(CamelFolderSummary *summary,
+						 const gchar *uid);
 GPtrArray *	camel_folder_summary_get_array	(CamelFolderSummary *summary);
 void		camel_folder_summary_free_array	(GPtrArray *array);
 
