@@ -1844,16 +1844,18 @@ imapx_store_get_folder_info_sync (CamelStore *store,
 			imapx_store->priv->last_refresh_time = time (NULL);
 
 			session = camel_service_ref_session (service);
-			description = g_strdup_printf (_("Retrieving folder list for '%s'"), camel_service_get_display_name (service));
+			if (session) {
+				description = g_strdup_printf (_("Retrieving folder list for '%s'"), camel_service_get_display_name (service));
 
-			camel_session_submit_job (
-				session, description, (CamelSessionCallback)
-				imapx_refresh_finfo,
-				g_object_ref (store),
-				(GDestroyNotify) g_object_unref);
+				camel_session_submit_job (
+					session, description, (CamelSessionCallback)
+					imapx_refresh_finfo,
+					g_object_ref (store),
+					(GDestroyNotify) g_object_unref);
 
-			g_object_unref (session);
-			g_free (description);
+				g_object_unref (session);
+				g_free (description);
+			}
 		}
 	}
 
@@ -3223,7 +3225,7 @@ camel_imapx_store_handle_list_response (CamelIMAPXStore *imapx_store,
 
 	/* Fabricate a CamelIMAPXNamespaceResponse if the server lacks the
 	 * NAMESPACE capability and this is the first LIST / LSUB response. */
-	if (camel_imapx_server_lack_capability (imapx_server, IMAPX_CAPABILITY_NAMESPACE)) {
+	if (CAMEL_IMAPX_LACK_CAPABILITY (camel_imapx_server_get_capability_info (imapx_server), NAMESPACE)) {
 		g_mutex_lock (&imapx_store->priv->namespaces_lock);
 		if (imapx_store->priv->namespaces == NULL) {
 			imapx_store->priv->namespaces = camel_imapx_namespace_response_faux_new (response);
@@ -3287,7 +3289,7 @@ camel_imapx_store_handle_lsub_response (CamelIMAPXStore *imapx_store,
 
 	/* Fabricate a CamelIMAPXNamespaceResponse if the server lacks the
 	 * NAMESPACE capability and this is the first LIST / LSUB response. */
-	if (camel_imapx_server_lack_capability (imapx_server, IMAPX_CAPABILITY_NAMESPACE)) {
+	if (CAMEL_IMAPX_LACK_CAPABILITY (camel_imapx_server_get_capability_info (imapx_server), NAMESPACE)) {
 		g_mutex_lock (&imapx_store->priv->namespaces_lock);
 		if (imapx_store->priv->namespaces == NULL) {
 			imapx_store->priv->namespaces = camel_imapx_namespace_response_faux_new (response);
